def draw_severity_icon(severity):
    """
    Visually generates a circular icon based on severity of the bug.
    Critical -> Red Color
    High -> Orange
    Medium -> Teal
    Low / Default -> Green

    Arguments
    severity : The importance of the bug detected by SonarQube
    """
    if severity == "CRITICAL":
        return "<font color='red' size='12'>&#9679;</font>"
    elif severity == "HIGH":
        return "<font color='orange' size='12'>&#9679;</font>"
    elif severity == "MEDIUM":
        return "<font color='teal' size='12'>&#9679;</font>"
    else:  # Low
        return "<font color='green' size='12'>&#9679;</font>"



import logging

import ipinfo


"""Integration class for handling IPInfo operations and geolocation lookups."""


class IPInfoIntegration:
    def __init__(self, api_key):
        """Initialize IPInfo integration with API key.
        @param api_key: The API key for IPInfo service
        """
        self.api_key = api_key
        self.default_country = "Default Location - North Korea"

    def get_user_geo_location(self):
        """Get the user's country based on their IP address.
        Returns:
            str: Two-letter country code or default location on error
        """
        try:
            handler = ipinfo.getHandler(self.api_key)
            details = handler.getDetails()
            user_country = details.country
            return user_country
        except Exception as e:
            logging.info(
                f"Something went wrong with IPInfo Integration with exception as :: {e}")
            return self.default_country



from reportlab.platypus import Paragraph, Table, SimpleDocTemplate
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.lib.units import inch
from reportlab.lib.pagesizes import letter
import datetime
import logging
from html import escape
import constants
from reports.utils.report_utils import draw_severity_icon
import styling
from styling import BRANDING_STYLE
from core import analyser
from core.utils.sonarqube_utils import get_duplication_map

normal_style = styling.NORMAL_STYLE
header_style = styling.HEADER_STYLE


def create_basic_project_details_table(project_name):
    """
    Creates the table for Basic Project Details

    Arguments:
    project_name : The name of the project for which PDF Report needs to be made

    Returns
    The tabular project structure consisting of basic details
    """

    data_project_basic_details = [
        [Paragraph(constants.TABLE_HEADER_PROJECT_NAME, header_style),
         Paragraph(constants.TABLE_HEADER_ANALYSIS, header_style),
         Paragraph(constants.TABLE_HEADER_BRANCH, header_style)]
    ]
    data_project_basic_details.append([
        Paragraph(project_name, normal_style),
        Paragraph(datetime.datetime.now().strftime(
            "%d/%m/%Y %H:%M:%S"), normal_style),
        Paragraph("Main / Master", normal_style)
    ])

    table_basic_details = Table(data_project_basic_details,
                                colWidths=[2 * inch, 3 * inch, 1 * inch, 2.5 * inch, 0.75 * inch])
    table_basic_details.setStyle(styling.TABLE_STYLE)
    return table_basic_details


def actual_table_content_data(component_list, fix_list, line_number_list, impact, issue_type_list):
    """
    Creates the table content for all the information regarding the vulnerabilities detected

    Arguments:
    components_list : The list of components where vulnerabilites have been detected
    fix_list : The fixes that needs to be done in the specific component
    line_number_list : The line number where fix needs to be made / vulnerability detected
    impact : The impact of the vulnerability
    issue_type_list : The type of vulnerability that has been detected

    Returns:
    The tabular data containing all information about the vulnerabilities detected
    """

    data = [
        [Paragraph(constants.TABLE_HEADER_SEVERITY, header_style),
         Paragraph(constants.TABLE_HEADER_DESCRIPTION, header_style),
         Paragraph(constants.TABLE_HEADER_TYPE, header_style),
         Paragraph(constants.TABLE_HEADER_FILE_NAME, header_style),
         Paragraph(constants.TABLE_HEADER_LINE_NUMBER, header_style)]
    ]

    # Table content
    logging.info("Total Issues detected are " + str(len(component_list)))
    for i in range(0, len(component_list)):
        severity_icon = draw_severity_icon(impact[i])
        description = fix_list[i]
        file_name = "/".join(component_list[i].split(":")[1:])
        line_number = str(line_number_list[i])
        issue_type_entry_table = issue_type_list[i]
        issue_type_entry_table = issue_type_entry_table.replace(
            "_", " ").title()

        data.append([
            Paragraph(severity_icon, normal_style),
            Paragraph(escape(description), normal_style),
            Paragraph(escape(issue_type_entry_table), normal_style),
            Paragraph(escape(file_name), normal_style),
            Paragraph(escape(line_number), normal_style),
        ])

    # Create table
    table = Table(data, colWidths=[0.75 * inch,
                  3 * inch, 1 * inch, 2.5 * inch, 0.75 * inch])
    table.setStyle(styling.TABLE_STYLE)
    return table


def duplication_table(duplication_map):
    """
    Creates the table for Duplication

    Arguments:
    duplication_map : The map of the file name and the number of duplicated lines

    Returns:
    The tabular data containing the duplication information
    """

    data = [
        [
            Paragraph(constants.TABLE_HEADER_FILE_NAME, header_style),
            Paragraph(constants.TABLE_HEADER_DUPLICATED_LINES, header_style)]
    ]
    for i, j in duplication_map.items():
        file_name = i
        duplicated_lines = j
        data.append([
            Paragraph(escape(file_name), normal_style),
            Paragraph(escape(duplicated_lines), normal_style),
        ])

    table = Table(data, colWidths=[3 * inch, 1 * inch])
    table.setStyle(styling.TABLE_STYLE)
    return table


def issue_summary_overview(bug_list, vulnerability_list, code_smell_list, duplication_list):
    """
    Creates the table content for the executive summary of the report

    Arguments:
    bug_list: Integer containing number of bugs found during the SonarQube analysis.
    vulnerability_list : Integer containing number of vulnerabilities found during the analysis.
    code_smell_list : Integer containing number of Code Smell found during the analysis.
    duplication list: Dummy as of now, always prints 0%. But ideally should give the duplication percentage.


    Returns:
    The tabular data containing the executive summary for the report.
    """

    data = [
        [Paragraph(constants.TABLE_HEADER_SUMMARY_BUGS, header_style),
         Paragraph(constants.TABLE_HEADER_SUMMARY_VULNERABILITIES, header_style),
         Paragraph(constants.TABLE_HEADER_SUMMARY_CODE_SMELLS, header_style),
         Paragraph(constants.TABLE_HEADER_SUMMARY_DUPLICATION_PER, header_style)
         ]
    ]

    # Table content

    data.append([
        Paragraph(escape(str(bug_list)), normal_style),
        Paragraph(escape(str(vulnerability_list)), normal_style),
        Paragraph(escape(str(code_smell_list)), normal_style),
        Paragraph(escape(str(duplication_list)), normal_style),
    ])

    # Create table
    table = Table(data, colWidths=[2 * inch, 2 * inch, 2 * inch, 2 * inch])
    table.setStyle(styling.TABLE_STYLE)
    return table


def create_issues_report(file_path, host_name, auth_token, project_name, protocol):
    response = analyser.get_reported_issues_by_sonarqube(
        host_name, auth_token, project_name, protocol)
    if (response == ""):
        logging.info(
            "We are sorry, we're having trouble generating your report")
        print("We are sorry, we're having trouble generating your report")
        return 0
    duplication_response = analyser.get_duplication_density(
        host_name, project_name, auth_token, protocol)
    duplication_map = get_duplication_map(
        host_name, project_name, auth_token, protocol)
    component_list, fix_list, line_number_list, impact, issue_type_list = analyser.get_issues_by_type(
        response, "CODE_SMELL")
    size_of_code_smell_list = len(component_list)
    component_list_vulnerability, fix_list_vulnerability, line_number_list_vulnerability, impact_vulnerability, issue_type_list_vulnerability = analyser.get_issues_by_type(
        response, "VULNERABILITY")
    size_of_vulnerability_list = len(component_list_vulnerability)
    component_list_bug, fix_list_bug, line_number_list_bug, imact_bug, issue_type_list_bug = analyser.get_issues_by_type(
        response, "BUG")
    size_of_bug_list = len(component_list_bug)

    component_list = component_list + component_list_vulnerability + component_list_bug
    fix_list = fix_list + fix_list_vulnerability + fix_list_bug
    line_number_list = line_number_list + \
        line_number_list_vulnerability + line_number_list_bug
    impact = impact + impact_vulnerability + imact_bug
    issue_type_list = issue_type_list + \
        issue_type_list_vulnerability + issue_type_list_bug

    doc = SimpleDocTemplate(file_path, pagesize=letter)

    # Custom styles
    title_style = styling.TITLE_STYLE
    subtitle_style = styling.SUBTITLE_STYLE
    elements = []
    elements.append(Paragraph(constants.CREATOR_CREDIT, BRANDING_STYLE))
    elements.append(Paragraph("", title_style))
    # Title
    elements.append(Paragraph(constants.DOCUMENT_HEADER, title_style))
    table_basic_details = create_basic_project_details_table(project_name)
    elements.append(table_basic_details)
    elements.append(Paragraph("", title_style))

    # Summary Section
    elements.append(
        Paragraph(constants.TABLE_HEADER_REPORT_SUMMARY, title_style))
    report_summary = issue_summary_overview(size_of_bug_list, size_of_vulnerability_list, size_of_code_smell_list,
                                            duplication_response)
    elements.append(report_summary)
    elements.append(Paragraph("", title_style))

    # Subtitle
    elements.append(Paragraph(constants.SUBHEADER_DOCUMENT, subtitle_style))
    table = actual_table_content_data(
        component_list, fix_list, line_number_list, impact, issue_type_list)
    elements.append(table)
    elements.append(Paragraph("", title_style))

    # Duplication
    if (len(duplication_map) > 0):
        elements.append(
            Paragraph(constants.SUBHEADER_DUPLICATION_DOCUMENT, subtitle_style))
        table = duplication_table(duplication_map=duplication_map)
        elements.append(table)

    doc.build(elements)



import click
from support import pick_random_support_message, warning_for_path_change
from dotenv import load_dotenv
from utils import general_utils
from core import analyser
load_dotenv()


@click.group()
def cli():
    pass


@click.command()
@click.option("--host", help="The host url where SonarQube server is running", required=True)
@click.option("--project", help="Name of the Project Key that we want to search for in SonarQube report ",
              required=True)
@click.option("--path", help="Path where we want to the PDF Report", required=False)
@click.option("--token", help="SonarQube Global Analysis Token", required=True)
@click.option("--protocol", type=click.Choice(["http", "https"], case_sensitive=False), required=False,
              help="The protocol that you want to enforce - HTTP or HTTPS")
def generatepdf(host, project, path, token, protocol):
    resolved_path = str(general_utils.check_and_validate_file_path(path))
    analyser.generate_final_report_and_transmit_to_sentry(
        resolved_path, host, project, token, protocol)
    print(pick_random_support_message())
    if path != resolved_path:
        print(warning_for_path_change(resolved_path))


cli.add_command(generatepdf)
if __name__ == "__main__":
    cli()



import logging
from pathlib import Path
from urllib.parse import urlparse
import constants


def check_and_validate_file_path(path):
    """
    Sanitise file path and checks if it is a valid directory to store the generated report.

    Arguments:
        path: The file path being provided by the user
    """
    if path is None:
        resolved_directory = Path.home() / "Downloads" / constants.FALLBACK_FILE_NAME
        return resolved_directory

    else:
        resolved_directory = ""
        result_ends_with_pdf = str(path).endswith(".pdf")
        if result_ends_with_pdf == False:
            resolved_directory = Path(path) / constants.FALLBACK_FILE_NAME
            return check_and_validate_file_path(resolved_directory)

        else:
            path_resolved = Path(path).resolve(strict=False)
            if path_resolved.parent.exists():
                logging.info(
                    "Since Path is valid and exists, we are not manipulating things at Server Side")
                resolved_directory = path
                return resolved_directory
            else:
                logging.info(
                    "Path does not exists, we will fallback to defaults")
                resolved_directory = Path.home() / "Downloads" / constants.FALLBACK_FILE_NAME
                return resolved_directory


def handle_protocol_for_every_communication(protocol, host):
    """
    Observed a lot of errors related to protocol since I was enforcing protocol based on custom logic which was incorrect. Figured out a temporary way to handle the protocol communication by adding extra parameter in click command which is optional currently. If this works out, I will promote this to required argument in the next major release - 2.5 onwards.

    Arguments:
        protocol - Only 2 available choices - http or https
    """

    if protocol == "" or protocol is None:
        logging.info(
            "No protocol has been supplied, hence we will go to fallback routes")
        protocol_type = ""
        if "localhost" in host:
            protocol_type = "http://"
        elif host.startswith("http://") or host.startswith("https://"):
            squandered_string = host.split(":")[0]
            protocol_type = squandered_string + "://"
        else:
            protocol_type = "http://"

        if "localhost" in host and host.startswith("https://"):
            protocol_type = "http://"
        return protocol_type

    else:
        logging.info(f"Protocol Supplied is {protocol}")
        protocol_type = protocol + "://"
        return protocol_type


def remove_protocol(url):
    """
    Remove the protocol from the URL
    """
    parsed_url = urlparse(url)
    return parsed_url.netloc + parsed_url.path



import logging
import requests
from requests.auth import HTTPBasicAuth
from utils import general_utils
from utils.general_utils import handle_protocol_for_every_communication


def get_duplication_map(host_name, project_name, auth_token, protocol):
    protocol_type = handle_protocol_for_every_communication(
        protocol, host_name)
    if host_name.startswith("http") or host_name.startswith("http"):
        host_name = general_utils.remove_protocol(host_name)
    DUPLICATION_URL = f"{protocol_type}{host_name}/api/measures/component_tree?component={project_name}&metricKeys=duplicated_lines"
    logging.info(f"Generated Duplication URL is :: {DUPLICATION_URL}")
    auth = HTTPBasicAuth(auth_token, "")
    duplication_response = requests.get(url=DUPLICATION_URL, auth=auth)
    if duplication_response.status_code != 200:
        logging.info(
            f"Something went wrong while fetching the duplication count. Recevied status code is : {duplication_response.status_code}")
        logging.info(
            "INFO : This would not impact your report generation but duplication table won't be visible to you")
        return {}
    else:
        try:
            duplication_map = {}
            duplication_response_json = duplication_response.json()
            duplication_files_component = duplication_response_json.get(
                "components")
            if duplication_files_component is None:
                logging.info(
                    "SonarQube API didn't respond with correct standard since Component Key is missing within Duplication Response")
                return {}
            else:
                if (len(duplication_files_component) > 0):
                    for i in range(0, len(duplication_files_component)):
                        duplication_file_measures = duplication_files_component[i].get(
                            "measures")
                        if duplication_file_measures is None:
                            logging.info(
                                "SonarQube API didn't respond in the correct format since it does not have a measure component in duplication API")

                        else:
                            if (len(duplication_file_measures) > 0):
                                duplicated_line_count = duplication_file_measures[0].get(
                                    "value")
                                if duplicated_line_count is not None:
                                    if int(duplicated_line_count) > 0:
                                        file_name = duplication_files_component[i]["path"]
                                        duplication_map.update(
                                            {file_name: duplicated_line_count})
                                        logging.info(duplication_map)
                                else:
                                    logging.info(
                                        "SonarQube API didn't respond in the correct format since we didn't get duplication count")
                            else:
                                logging.info(
                                    "SonarQube API didn't respond in the correct format since measures is not a list in duplication API")
                    return duplication_map
                else:
                    logging.info(
                        "SonarQube API didn't respond in correct format since Component is not returned as a list or size of component list is 0")
                    return {}
        except Exception:
            logging.info(
                "For some reasons, duplication map cannot be generated")
            return {}
